//
// MenuItemConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Navigation
{
    public abstract class MenuItemConfiguration : ConfigurationBase
    {
        protected MenuItemConfiguration(
            string configName,
            int indentLevel,
            string caption,
            string link)
            : base(configName, indentLevel)
        {
            this.caption = caption;
            this.link = link;
        }

        public const string Identifier = ConfigurationKeyword.MenuItem;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.MenuItem,
            ConfigurationKeyword.MenuItemType,
            ConfigurationKeyword.MenuItemCaption,
            ConfigurationKeyword.MenuItemLink,
            ConfigurationKeyword.MenuItemPage,
            ConfigurationKeyword.MenuItemDropdown
        };

        private readonly string caption;
        private readonly string link;

        public string Caption
        {
            get { return caption; }
        }

        public abstract ILinkable Page { get; }

        public virtual string Link
        {
            get { return link; }
        }

        public abstract string TypeText { get; }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                {
                    ConfigName,
                    TypeText,
                    Caption,
                    Link,
                    Page != null ? Page.Name : null,
                    null
                };
            }
        }

        public override string ToString()
        {
            return ConfigText(Params, Items);
        }

        public static MenuItemConfiguration Create(ConfigurationBlock config, IEnumerable<ILinkable> linkables)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            string itemName = GetOptionConfigName(options, ConfigurationKeyword.MenuItem, Identifier, true);
            string typeText =
                GetOptionString(options, ConfigurationKeyword.MenuItemType, Identifier, false, "Link").ToLower();
            string caption = GetOptionString(options, ConfigurationKeyword.MenuItemCaption, Identifier, false);
            string link = GetOptionString(options, ConfigurationKeyword.MenuItemLink, Identifier, false);
            string page = GetOptionString(options, ConfigurationKeyword.MenuItemPage, Identifier, false);
            string dropdown = GetOptionString(options, ConfigurationKeyword.MenuItemDropdown, Identifier, false);
            ILinkable linkable = null;

            MenuItemType type;

            if (dropdown != String.Empty)
                type = MenuItemType.Dropdown;
            else if (page != String.Empty)
                type = MenuItemType.Page;
            else if (typeText == String.Empty && link != String.Empty)
                type = MenuItemType.Link;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemTypeSpacer.ToLower())
                type = MenuItemType.Spacer;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemTypeHeader.ToLower())
                type = MenuItemType.Header;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemTypeName.ToLower())
                type = MenuItemType.Name;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemLink.ToLower())
                type = MenuItemType.Link;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemPage.ToLower())
                type = MenuItemType.Page;
            else if (typeText.Trim().ToLower() == ConfigurationKeyword.MenuItemDropdown.ToLower())
                type = MenuItemType.Dropdown;
            else
            {
                ConfigError(String.Format(ConfigurationMessage.UnknownMenuItem, typeText));
                type = MenuItemType.Unknown;
            }

            if (type == MenuItemType.Page)
            {
                linkable = linkables.SingleOrDefault(l => l.Name == page);
                if (linkable == null)
                {
                    ConfigError(String.Format(
                        ConfigurationMessage.MissingReference,
                        ConfigurationKeyword.MenuItemPage, page, Identifier));

                    ConfigError(ConfigurationMessage.ConfiguredPages);
                    foreach (ILinkable l in linkables)
                        ConfigError(l.Name);
                }
            }

            switch (type)
            {
                case MenuItemType.Spacer:
                    return new SpacerMenuItemConfiguration(String.Empty, config.IndentLevel);
                case MenuItemType.Header:
                    return new HeaderMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption);
                case MenuItemType.Name:
                    return new HeadMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption, link);
                case MenuItemType.Link:
                    return new LinkMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption, link);
                case MenuItemType.Page:
                    return new PageMenuItemConfiguration(String.Empty, config.IndentLevel, caption, linkable);
                case MenuItemType.Dropdown:
                    return DropdownItemConfiguration.Create(
                        itemName,
                        dropdown,
                        FindSingleBlock(ConfigurationKeyword.MenuItemDropdownContent, config),
                        linkables);
                default:
                    return null;
            }
        }
    }
}