//
// HostGrapPageConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Config.Destination;
using Ntp.Analyzer.Config.Graph;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Page
{
    public sealed class HostGraphPageConfiguration : GraphPageConfiguration
    {
        private HostGraphPageConfiguration(
            HostConfiguration server,
            string configName,
            int indentLevel,
            int frequency,
            bool initialRun,
            bool fixedRun,
            string theme,
            string title,
            HostPageConfiguration page,
            int linkIndex,
            DestinationCollection destinations,
            string link)
            : base(
                server,
                configName,
                indentLevel,
                frequency,
                initialRun,
                fixedRun,
                theme,
                title,
                linkIndex,
                destinations,
                link)
        {
            this.page = page;

            if (page != null)
            {
                page.AttachGraphPages(this);
            }
        }

        public const string Identifier = ConfigurationKeyword.HostGraphPage;

        private new const bool DefaultInitialRun = true;
        private new const bool DefaultFixedRun = false;
        private const int DefaultFrequency = 0;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.HostGraphPage,
            ConfigurationKeyword.Frequency,
            ConfigurationKeyword.InitialRun,
            ConfigurationKeyword.FixedRun,
            ConfigurationKeyword.PageTemplate,
            ConfigurationKeyword.PageTitle,
            ConfigurationKeyword.GraphPage,
            ConfigurationKeyword.LinkIndex,
            DirectoryCollection.Identifier,
            ConfigurationKeyword.PageLink
        };

        private readonly HostPageConfiguration page;

        public override IEnumerable<GraphSetConfiguration> GraphSets
        {
            get { return page.Graphs; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                var items = new List<object> {ConfigName};

                if (ShowDefault || (!ShowDefault && Frequency != DefaultFrequency))
                    items.Add(Frequency);

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(FixedRun);

                if (ShowDefault || (!ShowDefault && Theme.ToLower() != DefaultTheme.ToLower()))
                    items.Add(Theme);

                if (ShowDefault || (!ShowDefault && Title != DefaultTitle))
                    items.Add(Title);

                items.Add(page != null ? page.ConfigName : null);

                if (ShowDefault || (!ShowDefault && LinkIndex != DefaultLinkIndex))
                    items.Add(LinkIndex);

                items.Add(Destinations);

                if (ShowDefault || (!ShowDefault && Link != DefaultLink))
                    items.Add(Link);

                return items.ToArray();
            }
        }

        public string GetFileName(GraphSetConfiguration graphSet, GraphBaseConfiguration graph)
        {
            return graph.GetAltName(graphSet, null).
                Replace(".png", String.Empty).
                Replace('.', '-')
                   + ".html";
        }

        public string GetLink(GraphSetConfiguration graphSet, GraphBaseConfiguration graph)
        {
            return WebPath + Link +
                   graph.GetAltName(graphSet, null).
                       Replace(".png", String.Empty).
                       Replace('.', '-')
                   + ".html";
        }

        public override string ToString()
        {
            var list = new List<String> {ConfigurationKeyword.HostGraphPage};

            if (ShowDefault || (!ShowDefault && Frequency != DefaultFrequency))
                list.Add(ConfigurationKeyword.Frequency);

            if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                list.Add(ConfigurationKeyword.InitialRun);

            if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                list.Add(ConfigurationKeyword.FixedRun);

            if (ShowDefault || (!ShowDefault && Theme.ToLower() != DefaultTheme.ToLower()))
                list.Add(ConfigurationKeyword.PageTemplate);

            if (ShowDefault || (!ShowDefault && Title != DefaultTitle))
                list.Add(ConfigurationKeyword.PageTitle);

            list.Add(ConfigurationKeyword.GraphPage);

            if (ShowDefault || (!ShowDefault && LinkIndex != DefaultLinkIndex))
                list.Add(ConfigurationKeyword.LinkIndex);

            list.Add(DirectoryCollection.Identifier);

            if (ShowDefault || (!ShowDefault && Link != DefaultLink))
                list.Add(ConfigurationKeyword.PageLink);

            return ConfigText(list.ToArray(), Items);
        }

        public static HostGraphPageConfiguration Create(
            HostConfiguration server,
            ConfigurationBlock config,
            IEnumerable<HostPageConfiguration> hostPages)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            string hostPagesName = GetOptionString(
                options, ConfigurationKeyword.GraphPage, Identifier, false);
            HostPageConfiguration hostPage = null;

            if (hostPagesName != String.Empty)
            {
                hostPage = hostPages.SingleOrDefault(pp => pp.ConfigName == hostPagesName);

                if (hostPage == null)
                {
                    ConfigError(String.Format(
                        ConfigurationMessage.MissingReference,
                        ConfigurationKeyword.GraphPage, hostPagesName, Identifier
                        ));

                    ConfigError(ConfigurationMessage.ConfiguredGraphPages);
                    foreach (HostPageConfiguration hostPageConfig in hostPages)
                        ConfigError(hostPageConfig.ConfigName);
                }
            }

            return new HostGraphPageConfiguration(
                server,
                GetOptionConfigName(options, ConfigurationKeyword.HostGraphPage, Identifier, true),
                config.IndentLevel,
                GetOptionInteger(options, ConfigurationKeyword.Frequency, Identifier, false, DefaultFrequency),
                GetOptionBoolean(options, ConfigurationKeyword.InitialRun, Identifier, false, DefaultInitialRun),
                GetOptionBoolean(options, ConfigurationKeyword.FixedRun, Identifier, false, DefaultFixedRun),
                GetOptionString(options, ConfigurationKeyword.PageTemplate, Identifier, false, DefaultTheme),
                GetOptionString(options, ConfigurationKeyword.PageTitle, Identifier, false, DefaultTitle),
                hostPage,
                GetOptionInteger(options, ConfigurationKeyword.LinkIndex, Identifier, false, DefaultLinkIndex),
                DirectoryCollection.Create(server, config),
                GetOptionString(options, ConfigurationKeyword.PageLink, Identifier, false, null)
                );
        }
    }
}