//
// ReadingBulkConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Localize;
using Ntp.Config;
using Ntp.Process;

namespace Ntp.Analyzer.Config.Root
{
    public sealed class ReadingBulkConfiguration : JobConfiguration
    {
        private ReadingBulkConfiguration(
            string configName,
            int indentLevel,
            string name,
            DateTimeKind timeStamp,
            int frequency,
            bool initialRun,
            bool fixedRun)
            : base(
                configName,
                indentLevel,
                frequency,
                initialRun,
                fixedRun)
        {
            this.name = name;
            this.timeStamp = timeStamp;
        }

        public const string Identifier = ConfigurationKeyword.Reading;

        private const bool DefaultInitialRun = false;
        private const bool DefaultFixedRun = true;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.Reading,
            ConfigurationKeyword.ReadingName,
            ConfigurationKeyword.Frequency,
            ConfigurationKeyword.TimeStamp,
            ConfigurationKeyword.InitialRun,
            ConfigurationKeyword.FixedRun
        };

        private readonly string name;
        private readonly DateTimeKind timeStamp;

        private String[] InstanceParams
        {
            get
            {
                var items = new List<string>
                {
                    ConfigurationKeyword.Reading,
                    ConfigurationKeyword.ReadingName,
                    ConfigurationKeyword.Frequency,
                };

                if (ShowDefault || (!ShowDefault && TimeStamp != DefaultTimeStamp))
                    items.Add(ConfigurationKeyword.TimeStamp);

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(ConfigurationKeyword.InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(ConfigurationKeyword.FixedRun);

                return items.ToArray();
            }
        }

        public string Name
        {
            get { return name; }
        }

        public DateTimeKind TimeStamp
        {
            get { return timeStamp; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                var items = new List<object> {ConfigName, Name, Frequency};

                if (ShowDefault || (!ShowDefault && TimeStamp != DefaultTimeStamp))
                    items.Add(GetTimezoneString(TimeStamp));

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(FixedRun);

                return items.ToArray();
            }
        }

        public override string ToString()
        {
            return ConfigText(InstanceParams, Items);
        }

        public static ReadingBulkConfiguration Create(ConfigurationBlock config)
        {
            if (config == null)
                return null;

            Dictionary<String, String> options = GetOptions(Params, config);

            return new ReadingBulkConfiguration(
                GetOptionConfigName(options, ConfigurationKeyword.Reading, String.Empty, false),
                config.IndentLevel,
                GetOptionString(options, ConfigurationKeyword.ReadingName, Identifier, true),
                GetTimezone(options, ConfigurationKeyword.TimeStamp, Identifier),
                GetOptionInteger(options, ConfigurationKeyword.Frequency, Identifier, true),
                GetOptionBoolean(options, ConfigurationKeyword.InitialRun, Identifier, false, DefaultInitialRun),
                GetOptionBoolean(options, ConfigurationKeyword.FixedRun, Identifier, false, DefaultFixedRun)
                );
        }
    }
}